// streams.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

// Generates a newline character when passed to a stream.
Symbol* EOL = NULL;

// A string constant containing the character sequence
// necessary to generate a newline.
char* EOL_STRING = "\n";

// Denotes the standard input stream for the host language.
Input_Stream* STANDARD_INPUT = NULL;

// Denotes the standard output stream for the host language.
Output_Stream* STANDARD_OUTPUT = NULL;

// Denotes the standard warning stream for the host language.
Output_Stream* STANDARD_WARNING = NULL;

// Denotes the standard error stream for the host language.
Output_Stream* STANDARD_ERROR = NULL;

// Holds a string containing the single quote character.
String_Wrapper* SINGLE_QUOTE_STRING = NULL;

// Holds a string containing the single backquote character.
String_Wrapper* SINGLE_BQUOTE_STRING = NULL;

Cons* format_message_arguments(Cons* messageargumentstree, boolean trailingeolP) {
  { Cons* augmentedarguments = NIL;

    { Object* item = NULL;
      Cons* iter_000 = messageargumentstree;

      while (!(iter_000 == NIL)) {
        item = iter_000->value;
        iter_000 = iter_000->rest;
        if (stringP(item) ||
            (item == EOL)) {
          augmentedarguments = cons(item, augmentedarguments);
        }
        else {
          augmentedarguments = cons(SINGLE_BQUOTE_STRING, augmentedarguments);
          augmentedarguments = cons(item, augmentedarguments);
          augmentedarguments = cons(SINGLE_QUOTE_STRING, augmentedarguments);
        }
      }
    }
    if (trailingeolP) {
      augmentedarguments = cons(EOL, augmentedarguments);
    }
    return (augmentedarguments->reverse());
  }
}

Object* print_expander(Cons* body) {
  // Print arguments to the standard output stream.
  return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_OUTPUT, body->concatenate(NIL, 0)));
}

Object* inform_expander(Cons* body) {
  // Print informative message, placing non-string arguments in quotes,
  // and terminating with a newline.
  body = format_message_arguments(body, TRUE);
  return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_OUTPUT, body->concatenate(NIL, 0)));
}

Object* warn_expander(Cons* body) {
  // Signal warning message, placing non-string arguments in quotes.
  body = format_message_arguments(body, FALSE);
  if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_STREAMS_COMMON_LISP) {
    return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, KWD_STREAMS_WARN, body->concatenate(NIL, 0)));
  }
  else {
    return (listO(4, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_WARNING, wrap_string("Warning: "), body->concatenate(cons(SYM_STREAMS_STELLA_EOL, NIL), 0)));
  }
}

Object* signal_expander(Symbol* type, Cons* body) {
  // Signal error message, placing non-string arguments in quotes.
  if (type == SYM_STREAMS_STELLA_NULL) {
    type = SYM_STREAMS_STELLA_STELLA_EXCEPTION;
  }
  if ((body->length() == 1) &&
      isaP(body->value, SGT_STREAMS_STELLA_STRING_WRAPPER)) {
    return (listO(3, SYM_STREAMS_STELLA_SIGNAL_EXCEPTION, listO(3, SYM_STREAMS_STELLA_NEW, type, listO(3, KWD_STREAMS_MESSAGE, body->value, NIL)), NIL));
  }
  else {
    { Symbol* streamvar = local_gensym("STREAM");

      return (listO(5, SYM_STREAMS_STELLA_LET, cons(cons(streamvar, cons(listO(3, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM, NIL), NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, format_message_arguments(body, FALSE)->concatenate(NIL, 0)), listO(3, SYM_STREAMS_STELLA_SIGNAL_EXCEPTION, listO(3, SYM_STREAMS_STELLA_NEW, type, listO(3, KWD_STREAMS_MESSAGE, listO(3, SYM_STREAMS_STELLA_THE_STRING, streamvar, NIL), NIL)), NIL), NIL));
    }
  }
}

Object* error_expander(Cons* body) {
  // Signal error message, placing non-string arguments in quotes.
  return (listO(3, SYM_STREAMS_STELLA_SIGNAL, SYM_STREAMS_STELLA_STELLA_EXCEPTION, body->concatenate(NIL, 0)));
}

Object* continuable_error_expander(Cons* body) {
  // Signal error message, placing non-string arguments in quotes.
  body = format_message_arguments(body, FALSE);
  if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_STREAMS_JAVA) {
    if ((body->length() == 1) &&
        isaP(body->value, SGT_STREAMS_STELLA_STRING_WRAPPER)) {
      return (listO(4, SYM_STREAMS_STELLA_LET, cons(listO(3, SYM_STREAMS_STELLA_ERROR_MESSAGE_, wrap_string(((String_Wrapper*)(body->value))->wrapper_value), NIL), NIL), listO(4, SYM_STREAMS_STELLA_VERBATIM, KWD_STREAMS_JAVA, wrap_string("Native.continuableError(error_message_)"), NIL), NIL));
    }
    else {
      { Symbol* streamvar = local_gensym("STREAM");

        return (listO(5, SYM_STREAMS_STELLA_LET, cons(cons(streamvar, cons(listO(3, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM, NIL), NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, format_message_arguments(body, FALSE)->concatenate(NIL, 0)), listO(4, SYM_STREAMS_STELLA_LET, cons(listO(3, SYM_STREAMS_STELLA_ERROR_MESSAGE_, listO(3, SYM_STREAMS_STELLA_THE_STRING, streamvar, NIL), NIL), NIL), listO(4, SYM_STREAMS_STELLA_VERBATIM, KWD_STREAMS_JAVA, wrap_string("Native.continuableError(error_message_)"), NIL), NIL), NIL));
      }
    }
  }
  else {
    return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, KWD_STREAMS_CONTINUABLE_ERROR, body->concatenate(NIL, 0)));
  }
}

Object* signal_read_error_expander(Cons* body) {
  // Specialized version of `signal' that throws a READ-EXCEPTION.
  return (listO(3, SYM_STREAMS_STELLA_SIGNAL, SYM_STREAMS_STELLA_READ_EXCEPTION, body->concatenate(NIL, 0)));
}

Cons* format_walk_message_arguments(Cons* messageargumentstree) {
  { Cons_Iterator* it = messageargumentstree->allocate_iterator();

    while (it->nextP()) {
      { Object* item = it->value;

        if (subtype_of_symbolP(safe_primary_type(item))) {
          { Object* item_000 = item;
            Symbol* item = ((Symbol*)(item_000));

            if (!(item == EOL)) {
              it->value_setter(listO(3, SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE, item, NIL));
            }
          }
        }
        else {
        }
      }
    }
  }
  return (messageargumentstree);
}

Object* walk_error_expander(Cons* body) {
  { Cons* messagearguments = format_message_arguments(format_walk_message_arguments(body), FALSE);

    return (listO(5, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), cons(SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR, NIL), listO(5, SYM_STREAMS_STELLA_UNLESS, cons(SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT, wrap_string(">> ERROR: "), SYM_STREAMS_STELLA_STANDARD_ERROR, NIL), listO(5, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_ERROR, SYM_STREAMS_STELLA_EOL, wrap_string(" "), messagearguments->concatenate(listO(3, wrap_string("."), SYM_STREAMS_STELLA_EOL, NIL), 0)), NIL), NIL));
  }
}

Object* walk_inform_expander(Cons* body) {
  { Cons* messagearguments = format_message_arguments(format_walk_message_arguments(body), FALSE);

    return (listO(5, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), cons(SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING, NIL), listO(5, SYM_STREAMS_STELLA_UNLESS, cons(SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT, wrap_string(">> WARNING: "), SYM_STREAMS_STELLA_STANDARD_OUTPUT, NIL), listO(5, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_OUTPUT, SYM_STREAMS_STELLA_EOL, wrap_string(" "), messagearguments->concatenate(listO(3, wrap_string("."), SYM_STREAMS_STELLA_EOL, NIL), 0)), NIL), NIL));
  }
}

Object* walk_warn_expander(Cons* body) {
  { Cons* messagearguments = format_message_arguments(format_walk_message_arguments(body), FALSE);

    return (listO(5, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), cons(SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING, NIL), listO(5, SYM_STREAMS_STELLA_UNLESS, cons(SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT, wrap_string(">> WARNING: "), SYM_STREAMS_STELLA_STANDARD_WARNING, NIL), listO(5, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_WARNING, SYM_STREAMS_STELLA_EOL, wrap_string(" "), messagearguments->concatenate(listO(3, wrap_string("."), SYM_STREAMS_STELLA_EOL, NIL), 0)), NIL), NIL));
  }
}

Object* safety_expander(Integer_Wrapper* level, Object* test, Cons* body) {
  // Signal warning message, placing non-string arguments in quotes.
  if (level->wrapper_value > oSAFETYo.get()) {
    return (NULL);
  }
  body = format_message_arguments(body, FALSE);
  return (listO(4, SYM_STREAMS_STELLA_WHEN, listO(3, SYM_STREAMS_STELLA_NOT, test, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_STREAM, KWD_STREAMS_ERROR, wrap_string("Safety violation: "), body->concatenate(NIL, 0)), NIL));
}

Object* evaluation_error_expander(Cons* body) {
  { Symbol* streamvar = local_gensym("STREAM");

    return (listO(8, SYM_STREAMS_STELLA_LET, cons(cons(streamvar, cons(listO(3, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM, NIL), NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, listO(3, wrap_string("While evaluating '"), SYM_STREAMS_STELLA_oEVALUATIONTREEo, NIL)), listO(4, SYM_STREAMS_STELLA_WHEN, listO(3, SYM_STREAMS_STELLA_DEFINEDp, SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo, NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, listO(4, SYM_STREAMS_STELLA_EOL, wrap_string("' inside '"), SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo, NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, listO(3, wrap_string("':"), SYM_STREAMS_STELLA_EOL, NIL)), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, format_message_arguments(body, FALSE)->concatenate(NIL, 0)), listO(3, SYM_STREAMS_STELLA_SIGNAL_EXCEPTION, listO(5, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_EVALUATION_EXCEPTION, KWD_STREAMS_MESSAGE, listO(3, SYM_STREAMS_STELLA_THE_STRING, streamvar, NIL), NIL), NIL), NIL));
  }
}

Cons* format_string_message_arguments(Cons* messagearguments, boolean trailingeolP) {
  { Cons* formattedarguments = format_message_arguments(messagearguments, trailingeolP);

    { Cons_Iterator* it = formattedarguments->allocate_iterator();

      while (it->nextP()) {
        { Surrogate* test_value_000 = safe_primary_type(it->value);

          if (subtype_of_stringP(test_value_000)) {
          }
          else if (subtype_ofP(test_value_000, SGT_STREAMS_STELLA_LITERAL_WRAPPER)) {
            it->value_setter(wrap_string(stringify(it->value)));
          }
          else {
            if (it->value == SYM_STREAMS_STELLA_EOL) {
              it->value_setter(wrap_string(EOL_STRING));
            }
            else {
              it->value_setter(listO(3, SYM_STREAMS_STELLA_STRINGIFY, it->value, NIL));
            }
          }
        }
      }
    }
    return (formattedarguments);
  }
}

Object* message_to_string_expander(Cons* body) {
  { Cons* messagearguments = format_string_message_arguments(body, FALSE);
    Symbol* messagevar = local_gensym("MESSAGE");

    return (listO(5, SYM_STREAMS_STELLA_VRLET, cons(cons(messagevar, listO(3, SYM_STREAMS_STELLA_STRING, SYM_STREAMS_STELLA_NULL, NIL)), NIL), listO(4, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), listO(3, SYM_STREAMS_STELLA_SETQ, messagevar, cons(cons(SYM_STREAMS_STELLA_CONCATENATE, messagearguments->concatenate(NIL, 0)), NIL)), NIL), messagevar, NIL));
  }
}

Cons* expand_markup_tag_function(Cons* tag_and_stream, Cons* body, boolean xmlP) {
  { Object* tag = tag_and_stream->value;
    Symbol* tagvar = NULL;
    char* tagvalue = NULL;
    Object* stream = tag_and_stream->rest->value;
    Object* eolargument = tag_and_stream->rest->rest->value;
    boolean eolP = ((boolean)(eolargument)) &&
        coerce_wrapped_boolean_to_boolean(coerce_to_boolean(eolargument));

    if (body == NIL) {
      { Surrogate* test_value_000 = safe_primary_type(tag);

        if (subtype_of_stringP(test_value_000)) {
          { Object* tag_000 = tag;
            String_Wrapper* tag = ((String_Wrapper*)(tag_000));

            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrap_string(string_concatenate("<", tag->wrapper_value, 1, (xmlP ? (char*)"/>" : (char*)">"))), cons(SYM_STREAMS_STELLA_EOL, NIL))));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrap_string(string_concatenate("<", tag->wrapper_value, 1, (xmlP ? (char*)"/>" : (char*)">"))), NIL)));
            }
          }
        }
        else if (subtype_ofP(test_value_000, SGT_STREAMS_STELLA_OBJECT)) {
          { Object* tag_001 = tag;
            Object* tag = tag_001;

            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrap_string("<"), tag, cons((xmlP ? wrap_string("/>") : wrap_string(">")), cons(SYM_STREAMS_STELLA_EOL, NIL)))));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrap_string("<"), tag, cons((xmlP ? wrap_string("/>") : wrap_string(">")), NIL))));
            }
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
    else {
      { Surrogate* test_value_001 = safe_primary_type(tag);

        if (subtype_of_stringP(test_value_001)) {
          { Object* tag_002 = tag;
            String_Wrapper* tag = ((String_Wrapper*)(tag_002));

            tagvalue = tag->wrapper_value;
            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrap_string(string_concatenate("<", tagvalue, 1, ">")), NIL)), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrap_string(string_concatenate("</", string_subsequence(tagvalue, 0, string_position(tagvalue, ' ', 0)), 1, ">")), cons(SYM_STREAMS_STELLA_EOL, NIL))), NIL), 0)));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrap_string(string_concatenate("<", tagvalue, 1, ">")), NIL)), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrap_string(string_concatenate("</", string_subsequence(tagvalue, 0, string_position(tagvalue, ' ', 0)), 1, ">")), NIL)), NIL), 0)));
            }
          }
        }
        else if (subtype_of_symbolP(test_value_001)) {
          { Object* tag_003 = tag;
            Symbol* tag = ((Symbol*)(tag_003));

            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrap_string("<"), tag, cons(wrap_string(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(5, wrap_string("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tag, listO(3, wrap_integer(0), listO(3, SYM_STREAMS_STELLA_POSITION, tag, listO(3, wrap_character(' '), wrap_integer(0), NIL)), NIL)), wrap_string(">"), SYM_STREAMS_STELLA_EOL, NIL)), NIL), 0)));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrap_string("<"), tag, cons(wrap_string(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(4, wrap_string("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tag, listO(3, wrap_integer(0), listO(3, SYM_STREAMS_STELLA_POSITION, tag, listO(3, wrap_character(' '), wrap_integer(0), NIL)), NIL)), wrap_string(">"), NIL)), NIL), 0)));
            }
          }
        }
        else if (subtype_ofP(test_value_001, SGT_STREAMS_STELLA_OBJECT)) {
          { Object* tag_004 = tag;
            Object* tag = tag_004;

            tagvar = gensym("TAG");
            if (eolP) {
              return (listO(4, SYM_STREAMS_STELLA_LET, cons(cons(tagvar, listO(3, SYM_STREAMS_STELLA_STRING, tag, NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrap_string("<"), tagvar, cons(wrap_string(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(5, wrap_string("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tagvar, listO(3, wrap_integer(0), listO(3, SYM_STREAMS_STELLA_POSITION, tagvar, listO(3, wrap_character(' '), wrap_integer(0), NIL)), NIL)), wrap_string(">"), SYM_STREAMS_STELLA_EOL, NIL)), NIL), 0)));
            }
            else {
              return (listO(4, SYM_STREAMS_STELLA_LET, cons(cons(tagvar, listO(3, SYM_STREAMS_STELLA_STRING, tag, NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrap_string("<"), tagvar, cons(wrap_string(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(4, wrap_string("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tagvar, listO(3, wrap_integer(0), listO(3, SYM_STREAMS_STELLA_POSITION, tagvar, listO(3, wrap_character(' '), wrap_integer(0), NIL)), NIL)), wrap_string(">"), NIL)), NIL), 0)));
            }
          }
        }
        else {
          { Output_String_Stream* stream_001 = new_output_string_stream();

            *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_001->the_string_reader());
          }
        }
      }
    }
  }
}

Object* with_html_tag_expander(Cons* tag_and_stream, Cons* body) {
  return (expand_markup_tag_function(tag_and_stream, body, FALSE));
}

Object* with_xml_tag_expander(Cons* tag_and_stream, Cons* body) {
  return (expand_markup_tag_function(tag_and_stream, body, TRUE));
}

boolean terminate_output_streamP(Output_Stream* self) {
  { ostream* native_stream = self->native_stream;

    if (!(native_stream == NULL)) {
      delete native_stream;
    }
    self->native_stream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    return (TRUE);
  }
}

boolean terminate_input_streamP(Input_Stream* self) {
  { istream* native_stream = self->native_stream;

    if (!(native_stream == NULL)) {
      delete native_stream;
    }
    self->native_stream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    return (TRUE);
  }
}

// List of file streams that are currently open.
List* oOPEN_FILE_STREAMSo = NULL;

void initialize_file_output_stream(Output_File_Stream* self) {
  { char* filename = translate_logical_pathname(self->filename);
    char* ifexistsaction = self->if_exists_action->symbol_name;

    self->native_stream = (new ofstream(filename));
    if (self->native_stream == NULL) {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "initialize-file-output-stream: Could not open " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    self->state = KWD_STREAMS_OPEN;
    oOPEN_FILE_STREAMSo->push(self);
  }
}

boolean terminate_file_output_streamP(Output_File_Stream* self) {
  { ostream* native_stream = self->native_stream;

    if (!(native_stream == NULL)) {
      delete native_stream;
    }
    self->native_stream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    oOPEN_FILE_STREAMSo->remove(self);
    return (TRUE);
  }
}

void initialize_file_input_stream(Input_File_Stream* self) {
  { char* filename = translate_logical_pathname(self->filename);
    char* ifnotexistsaction = self->if_not_exists_action->symbol_name;

    self->native_stream = (new ifstream(filename));
    if (self->native_stream == NULL) {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "initialize-file-input-stream: Could not open " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    self->state = KWD_STREAMS_OPEN;
    oOPEN_FILE_STREAMSo->push(self);
  }
}

boolean terminate_file_input_streamP(Input_File_Stream* self) {
  { istream* native_stream = self->native_stream;

    if (!(native_stream == NULL)) {
      delete native_stream;
    }
    self->native_stream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    oOPEN_FILE_STREAMSo->remove(self);
    return (TRUE);
  }
}

void initialize_string_output_stream(Output_String_Stream* self) {
  self->native_stream = (new (GC) ostrstream());
}

char* Output_String_Stream::the_string_reader() {
  { Output_String_Stream* self = this;

    { ostream* native_stream = self->native_stream;
      char* string = NULL;

      string = ((ostrstream*)native_stream)->str();
      if (string == NULL) {
        string = "";
      }
      return (string);
    }
  }
}

void initialize_string_input_stream(Input_String_Stream* self) {
  { char* string = self->the_string;

    self->native_stream = (new (GC) istrstream(string));
  }
}

void close_stream(Stream* self) {
  // Close the stream `self'.
  { Surrogate* test_value_000 = safe_primary_type(self);

    if (subtype_ofP(test_value_000, SGT_STREAMS_STELLA_FILE_INPUT_STREAM)) {
      { Stream* self_000 = self;
        Input_File_Stream* self = ((Input_File_Stream*)(self_000));

        terminate_file_input_streamP(self);
      }
    }
    else if (subtype_ofP(test_value_000, SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM)) {
      { Stream* self_001 = self;
        Output_File_Stream* self = ((Output_File_Stream*)(self_001));

        terminate_file_output_streamP(self);
      }
    }
    else if (subtype_ofP(test_value_000, SGT_STREAMS_STELLA_INPUT_STREAM)) {
      { Stream* self_002 = self;
        Input_Stream* self = ((Input_Stream*)(self_002));

        terminate_input_streamP(self);
      }
    }
    else if (subtype_ofP(test_value_000, SGT_STREAMS_STELLA_OUTPUT_STREAM)) {
      { Stream* self_003 = self;
        Output_Stream* self = ((Output_Stream*)(self_003));

        terminate_output_streamP(self);
      }
    }
    else {
    }
  }
}

void flush_output(Output_Stream* self) {
  // Flush all buffered output of `self'.
  { ostream* native_stream = self->native_stream;

    flush(*native_stream);
  }
}

void close_all_files() {
  // Close all currently open file streams.  Use for emergencies or for cleanup.
  { List* openstreams = oOPEN_FILE_STREAMSo->copy();

    { Stream* stream = NULL;
      Cons* iter_000 = openstreams->the_cons_list;

      while (!(iter_000 == NIL)) {
        stream = ((Stream*)(iter_000->value));
        iter_000 = iter_000->rest;
        close_stream(stream);
      }
    }
    openstreams->free();
  }
}

Input_Stream* open_network_stream(char* host, int port, Output_Stream*& _return1) {
  // Open a TCP/IP network stream to `host' at `port' and return the result
  // as an input/output stream pair.
  { istream* in = NULL;
    ostream* out = NULL;

    throw *new_stella_exception("Network streams not yet implemented for this target language");
    { Input_Stream* self_001 = new_input_stream();

      self_001->native_stream = in;
      self_001->state = KWD_STREAMS_OPEN;
      { Input_Stream* value_000 = self_001;

        { Output_Stream* self_002 = new_output_stream();

          self_002->native_stream = out;
          self_002->state = KWD_STREAMS_OPEN;
          { Output_Stream* value_001 = self_002;

            _return1 = value_001;
            return (value_000);
          }
        }
      }
    }
  }
}

char* oHTML_QUOTED_CHARACTERSo = "><&\"";

boolean html_character_needs_quotingP(char ch) {
  return (string_memberP(oHTML_QUOTED_CHARACTERSo, ch));
}

void write_html_character_quoting_special_characters(ostream* stream, char ch) {
  switch (ch) {
    case '>': 
      *(stream) << "&gt;";
    break;
    case '<': 
      *(stream) << "&lt;";
    break;
    case '&': 
      *(stream) << "&amp;";
    break;
    case '"': 
      *(stream) << "&quot;";
    break;
    default:
      *(stream) << ch;
    break;
  }
  return;
}

void write_html_quoting_special_characters(ostream* stream, char* input) {
  { char ch = NULL_CHARACTER;
    char* vector_000 = input;
    int index_000 = 0;
    int length_000 = strlen(vector_000);

    while (index_000 < length_000) {
      ch = vector_000[index_000];
      index_000 = index_000 + 1;
      write_html_character_quoting_special_characters(stream, ch);
    }
  }
  return;
}

void write_html_escaping_url_special_characters(ostream* stream, char* input) {
  { char ch = NULL_CHARACTER;
    char* vector_000 = input;
    int index_000 = 0;
    int length_000 = strlen(vector_000);

    while (index_000 < length_000) {
      ch = vector_000[index_000];
      index_000 = index_000 + 1;
      switch (ch) {
        case ' ': 
          *(stream) << "%20";
        break;
        case '?': 
          *(stream) << "%3F";
        break;
        case '#': 
          *(stream) << "%23";
        break;
        case '=': 
          *(stream) << "%3D";
        break;
        case '&': 
          *(stream) << "%26";
        break;
        case '%': 
          *(stream) << "%25";
        break;
        case '+': 
          *(stream) << "%2B";
        break;
        case '<': 
          *(stream) << "%3C";
        break;
        case '>': 
          *(stream) << "%3E";
        break;
        case '/': 
          *(stream) << "%2F";
        break;
        case ':': 
          *(stream) << "%3A";
        break;
        case '"': 
          *(stream) << "%22";
        break;
        default:
          *(stream) << ch;
        break;
      }
    }
  }
  return;
}

Key_Value_List* oHTML_ESCAPE_TABLEo = NULL;

char* unescape_html_string(char* input) {
  // Replaces HTML escape sequences such as &amp; with their associated
  // characters.
  if (!(string_position(input, '&', 0) != NULL_INTEGER)) {
    return (input);
  }
  { Output_String_Stream* out = new_output_string_stream();
    char* escape_string = NULL;

    { char ch = NULL_CHARACTER;
      char* vector_000 = input;
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        switch (ch) {
          case '&': 
            if (escape_string != NULL) {
              throw *new_stella_exception("& inside escape sequence!");
            }
            escape_string = "";
          break;
          case ';': 
            if (escape_string != NULL) {
              {
                if (((boolean)(oHTML_ESCAPE_TABLEo->lookup(wrap_string(escape_string))))) {
                  *(out->native_stream) << ((Character_Wrapper*)(oHTML_ESCAPE_TABLEo->lookup(wrap_string(escape_string))))->wrapper_value;
                }
                else {
                  *(out->native_stream) << "&" << escape_string << ";";
                }
                escape_string = NULL;
              }
            }
            else {
              *(out->native_stream) << ";";
            }
          break;
          default:
            if (escape_string != NULL) {
              escape_string = string_concatenate(escape_string, make_string(1, ch), 0);
            }
            else {
              *(out->native_stream) << ch;
            }
          break;
        }
      }
    }
    return (out->the_string_reader());
  }
}

Object* access_stream_iterator_slot_value(Stream_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_STREAMS_STELLA_THE_STREAM) {
    if (setvalueP) {
      self->the_stream = ((Input_Stream*)(value));
    }
    else {
      value = self->the_stream;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

S_Expression_Iterator* new_s_expression_iterator() {
  { S_Expression_Iterator* self = NULL;

    self = new S_Expression_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->the_stream = NULL;
    return (self);
  }
}

void S_Expression_Iterator::free() {
  { S_Expression_Iterator* self = this;

    if (terminate_stream_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* S_Expression_Iterator::primary_type() {
  { S_Expression_Iterator* self = this;

    return (SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR);
  }
}

Line_Iterator* new_line_iterator() {
  { Line_Iterator* self = NULL;

    self = new Line_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->the_stream = NULL;
    return (self);
  }
}

void Line_Iterator::free() {
  { Line_Iterator* self = this;

    if (terminate_stream_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Line_Iterator::primary_type() {
  { Line_Iterator* self = this;

    return (SGT_STREAMS_STELLA_LINE_ITERATOR);
  }
}

Character_Iterator* new_character_iterator() {
  { Character_Iterator* self = NULL;

    self = new Character_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL_CHARACTER;
    self->the_stream = NULL;
    return (self);
  }
}

void Character_Iterator::free() {
  { Character_Iterator* self = this;

    if (terminate_stream_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Character_Iterator::primary_type() {
  { Character_Iterator* self = this;

    return (SGT_STREAMS_STELLA_CHARACTER_ITERATOR);
  }
}

boolean terminate_stream_iteratorP(Stream_Iterator* self) {
  if (((boolean)(self->the_stream))) {
    self->the_stream->free();
  }
  return (TRUE);
}

S_Expression_Iterator* s_expressions(Input_Stream* stream) {
  { S_Expression_Iterator* self_000 = new_s_expression_iterator();

    self_000->the_stream = stream;
    { S_Expression_Iterator* value_000 = self_000;

      return (value_000);
    }
  }
}

boolean S_Expression_Iterator::nextP() {
  { S_Expression_Iterator* self = this;

    self->first_iterationP = FALSE;
    { Object* object = NULL;
      boolean eofP = FALSE;

      object = read_s_expression(self->the_stream, eofP);
      if (eofP) {
        close_stream(self->the_stream);
        return (FALSE);
      }
      else {
        self->value = object;
        return (TRUE);
      }
    }
  }
}

Line_Iterator* lines(Input_Stream* stream) {
  { Line_Iterator* self_000 = new_line_iterator();

    self_000->the_stream = stream;
    { Line_Iterator* value_000 = self_000;

      return (value_000);
    }
  }
}

boolean Line_Iterator::nextP() {
  { Line_Iterator* self = this;

    self->first_iterationP = FALSE;
    { char* line = NULL;
      boolean eofP = FALSE;

      line = read_line(self->the_stream, eofP);
      if (eofP) {
        close_stream(self->the_stream);
        return (FALSE);
      }
      else {
        self->value = line;
        return (TRUE);
      }
    }
  }
}

Character_Iterator* characters(Input_Stream* stream) {
  { Character_Iterator* self_000 = new_character_iterator();

    self_000->the_stream = stream;
    { Character_Iterator* value_000 = self_000;

      return (value_000);
    }
  }
}

boolean Character_Iterator::nextP() {
  { Character_Iterator* self = this;

    self->first_iterationP = FALSE;
    { char renamed_Char = NULL_CHARACTER;
      boolean eofP = FALSE;

      renamed_Char = read_character(self->the_stream, eofP);
      if (eofP) {
        close_stream(self->the_stream);
        return (FALSE);
      }
      else {
        self->value = renamed_Char;
        return (TRUE);
      }
    }
  }
}

void help_startup_streams1() {
  {
    SYM_STREAMS_STELLA_EOL = ((Symbol*)(intern_rigid_symbol_wrt_module("EOL", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-STREAM", NULL, 0)));
    SYM_STREAMS_STELLA_STANDARD_OUTPUT = ((Symbol*)(intern_rigid_symbol_wrt_module("STANDARD-OUTPUT", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_INFORM = ((Symbol*)(intern_rigid_symbol_wrt_module("INFORM", NULL, 0)));
    SYM_STREAMS_STELLA_INFORM_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("INFORM-EXPANDER", NULL, 0)));
    KWD_STREAMS_COMMON_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMON-LISP", NULL, 2)));
    KWD_STREAMS_WARN = ((Keyword*)(intern_rigid_symbol_wrt_module("WARN", NULL, 2)));
    SYM_STREAMS_STELLA_STANDARD_WARNING = ((Symbol*)(intern_rigid_symbol_wrt_module("STANDARD-WARNING", NULL, 0)));
    SYM_STREAMS_STELLA_WARN = ((Symbol*)(intern_rigid_symbol_wrt_module("WARN", NULL, 0)));
    SYM_STREAMS_STELLA_WARN_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("WARN-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SYM_STREAMS_STELLA_STELLA_EXCEPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("STELLA-EXCEPTION", NULL, 0)));
    SGT_STREAMS_STELLA_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING-WRAPPER", NULL, 1)));
    SYM_STREAMS_STELLA_SIGNAL_EXCEPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL-EXCEPTION", NULL, 0)));
    SYM_STREAMS_STELLA_NEW = ((Symbol*)(intern_rigid_symbol_wrt_module("NEW", NULL, 0)));
    KWD_STREAMS_MESSAGE = ((Keyword*)(intern_rigid_symbol_wrt_module("MESSAGE", NULL, 2)));
    SYM_STREAMS_STELLA_LET = ((Symbol*)(intern_rigid_symbol_wrt_module("LET", NULL, 0)));
    SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("OUTPUT-STRING-STREAM", NULL, 0)));
    SYM_STREAMS_STELLA_THE_STRING = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-STRING", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_ERROR_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("ERROR-EXPANDER", NULL, 0)));
    KWD_STREAMS_JAVA = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA", NULL, 2)));
    SYM_STREAMS_STELLA_ERROR_MESSAGE_ = ((Symbol*)(intern_rigid_symbol_wrt_module("ERROR_MESSAGE_", NULL, 0)));
    SYM_STREAMS_STELLA_VERBATIM = ((Symbol*)(intern_rigid_symbol_wrt_module("VERBATIM", NULL, 0)));
    KWD_STREAMS_CONTINUABLE_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTINUABLE-ERROR", NULL, 2)));
    SYM_STREAMS_STELLA_CONTINUABLE_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("CONTINUABLE-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("CONTINUABLE-ERROR-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_READ_EXCEPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("READ-EXCEPTION", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_READ_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL-READ-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL-READ-ERROR-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE = ((Symbol*)(intern_rigid_symbol_wrt_module("DE-UGLIFY-PARSE-TREE", NULL, 0)));
    SYM_STREAMS_STELLA_SPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIAL", NULL, 0)));
    SYM_STREAMS_STELLA_oPRINTREADABLYpo = ((Symbol*)(intern_rigid_symbol_wrt_module("*PRINTREADABLY?*", NULL, 0)));
    SYM_STREAMS_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL-TRANSLATION-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_UNLESS = ((Symbol*)(intern_rigid_symbol_wrt_module("UNLESS", NULL, 0)));
    SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp = ((Symbol*)(intern_rigid_symbol_wrt_module("SUPPRESS-WARNINGS?", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-ERROR-CONTEXT", NULL, 0)));
    SYM_STREAMS_STELLA_STANDARD_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("STANDARD-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("WALK-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("WALK-ERROR-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNAL-TRANSLATION-WARNING", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_INFORM = ((Symbol*)(intern_rigid_symbol_wrt_module("WALK-INFORM", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("WALK-INFORM-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_WARN = ((Symbol*)(intern_rigid_symbol_wrt_module("WALK-WARN", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_WARN_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("WALK-WARN-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("WHEN", NULL, 0)));
    SYM_STREAMS_STELLA_NOT = ((Symbol*)(intern_rigid_symbol_wrt_module("NOT", NULL, 0)));
    KWD_STREAMS_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 2)));
    SYM_STREAMS_STELLA_SAFETY = ((Symbol*)(intern_rigid_symbol_wrt_module("SAFETY", NULL, 0)));
    SYM_STREAMS_STELLA_SAFETY_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("SAFETY-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_oEVALUATIONTREEo = ((Symbol*)(intern_rigid_symbol_wrt_module("*EVALUATIONTREE*", NULL, 0)));
    SYM_STREAMS_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
    SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo = ((Symbol*)(intern_rigid_symbol_wrt_module("*EVALUATIONPARENTTREE*", NULL, 0)));
    SYM_STREAMS_STELLA_EVALUATION_EXCEPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("EVALUATION-EXCEPTION", NULL, 0)));
    SYM_STREAMS_STELLA_EVALUATION_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("EVALUATION-ERROR", NULL, 0)));
  }
}

void help_startup_streams2() {
  {
    SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("EVALUATION-ERROR-EXPANDER", NULL, 0)));
    SGT_STREAMS_STELLA_LITERAL_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("LITERAL-WRAPPER", NULL, 1)));
    SYM_STREAMS_STELLA_STRINGIFY = ((Symbol*)(intern_rigid_symbol_wrt_module("STRINGIFY", NULL, 0)));
    SYM_STREAMS_STELLA_VRLET = ((Symbol*)(intern_rigid_symbol_wrt_module("VRLET", NULL, 0)));
    SYM_STREAMS_STELLA_STRING = ((Symbol*)(intern_rigid_symbol_wrt_module("STRING", NULL, 0)));
    SYM_STREAMS_STELLA_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SETQ", NULL, 0)));
    SYM_STREAMS_STELLA_CONCATENATE = ((Symbol*)(intern_rigid_symbol_wrt_module("CONCATENATE", NULL, 0)));
    SYM_STREAMS_STELLA_MESSAGE_TO_STRING = ((Symbol*)(intern_rigid_symbol_wrt_module("MESSAGE-TO-STRING", NULL, 0)));
    SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("MESSAGE-TO-STRING-EXPANDER", NULL, 0)));
    SGT_STREAMS_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    SYM_STREAMS_STELLA_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("PROGN", NULL, 0)));
    SYM_STREAMS_STELLA_SUBSEQUENCE = ((Symbol*)(intern_rigid_symbol_wrt_module("SUBSEQUENCE", NULL, 0)));
    SYM_STREAMS_STELLA_POSITION = ((Symbol*)(intern_rigid_symbol_wrt_module("POSITION", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_HTML_TAG = ((Symbol*)(intern_rigid_symbol_wrt_module("WITH-HTML-TAG", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("WITH-HTML-TAG-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_XML_TAG = ((Symbol*)(intern_rigid_symbol_wrt_module("WITH-XML-TAG", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("WITH-XML-TAG-EXPANDER", NULL, 0)));
    KWD_STREAMS_CLOSED = ((Keyword*)(intern_rigid_symbol_wrt_module("CLOSED", NULL, 2)));
    KWD_STREAMS_OPEN = ((Keyword*)(intern_rigid_symbol_wrt_module("OPEN", NULL, 2)));
    SGT_STREAMS_STELLA_FILE_INPUT_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("FILE-INPUT-STREAM", NULL, 1)));
    SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("FILE-OUTPUT-STREAM", NULL, 1)));
    SGT_STREAMS_STELLA_INPUT_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("INPUT-STREAM", NULL, 1)));
    SGT_STREAMS_STELLA_OUTPUT_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("OUTPUT-STREAM", NULL, 1)));
    SYM_STREAMS_STELLA_THE_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-STREAM", NULL, 0)));
    SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("S-EXPRESSION-ITERATOR", NULL, 1)));
    SGT_STREAMS_STELLA_LINE_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("LINE-ITERATOR", NULL, 1)));
    SGT_STREAMS_STELLA_CHARACTER_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("CHARACTER-ITERATOR", NULL, 1)));
    SYM_STREAMS_STELLA_STARTUP_STREAMS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-STREAMS", NULL, 0)));
    SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_streams3() {
  {
    define_function_object("FORMAT-MESSAGE-ARGUMENTS", "(DEFUN (FORMAT-MESSAGE-ARGUMENTS CONS) ((MESSAGEARGUMENTSTREE CONS) (TRAILINGEOL? BOOLEAN)))", ((cpp_function_code)(&format_message_arguments)), NULL);
    define_function_object("PRINT-EXPANDER", "(DEFUN (PRINT-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Print arguments to the standard output stream.\")", ((cpp_function_code)(&print_expander)), NULL);
    define_function_object("INFORM-EXPANDER", "(DEFUN (INFORM-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Print informative message, placing non-string arguments in quotes,\nand terminating with a newline.\")", ((cpp_function_code)(&inform_expander)), NULL);
    define_function_object("WARN-EXPANDER", "(DEFUN (WARN-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal warning message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&warn_expander)), NULL);
    define_function_object("SIGNAL-EXPANDER", "(DEFUN (SIGNAL-EXPANDER OBJECT) ((TYPE SYMBOL) |&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal error message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&signal_expander)), NULL);
    define_function_object("ERROR-EXPANDER", "(DEFUN (ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal error message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&error_expander)), NULL);
    define_function_object("CONTINUABLE-ERROR-EXPANDER", "(DEFUN (CONTINUABLE-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal error message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&continuable_error_expander)), NULL);
    define_function_object("SIGNAL-READ-ERROR-EXPANDER", "(DEFUN (SIGNAL-READ-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Specialized version of `signal' that throws a READ-EXCEPTION.\")", ((cpp_function_code)(&signal_read_error_expander)), NULL);
    define_function_object("FORMAT-WALK-MESSAGE-ARGUMENTS", "(DEFUN (FORMAT-WALK-MESSAGE-ARGUMENTS CONS) ((MESSAGEARGUMENTSTREE CONS)))", ((cpp_function_code)(&format_walk_message_arguments)), NULL);
    define_function_object("WALK-ERROR-EXPANDER", "(DEFUN (WALK-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&walk_error_expander)), NULL);
    define_function_object("WALK-INFORM-EXPANDER", "(DEFUN (WALK-INFORM-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&walk_inform_expander)), NULL);
    define_function_object("WALK-WARN-EXPANDER", "(DEFUN (WALK-WARN-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&walk_warn_expander)), NULL);
    define_function_object("SAFETY-EXPANDER", "(DEFUN (SAFETY-EXPANDER OBJECT) ((LEVEL INTEGER-WRAPPER) (TEST OBJECT) |&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal warning message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&safety_expander)), NULL);
    define_function_object("EVALUATION-ERROR-EXPANDER", "(DEFUN (EVALUATION-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&evaluation_error_expander)), NULL);
    define_function_object("FORMAT-STRING-MESSAGE-ARGUMENTS", "(DEFUN (FORMAT-STRING-MESSAGE-ARGUMENTS CONS) ((MESSAGEARGUMENTS CONS) (TRAILINGEOL? BOOLEAN)))", ((cpp_function_code)(&format_string_message_arguments)), NULL);
    define_function_object("MESSAGE-TO-STRING-EXPANDER", "(DEFUN (MESSAGE-TO-STRING-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&message_to_string_expander)), NULL);
    define_function_object("EXPAND-MARKUP-TAG-FUNCTION", "(DEFUN (EXPAND-MARKUP-TAG-FUNCTION CONS) ((TAG-AND-STREAM CONS) (BODY CONS) (XML? BOOLEAN)))", ((cpp_function_code)(&expand_markup_tag_function)), NULL);
    define_function_object("WITH-HTML-TAG-EXPANDER", "(DEFUN (WITH-HTML-TAG-EXPANDER OBJECT) ((TAG-AND-STREAM CONS) |&BODY| (BODY CONS)))", ((cpp_function_code)(&with_html_tag_expander)), NULL);
    define_function_object("WITH-XML-TAG-EXPANDER", "(DEFUN (WITH-XML-TAG-EXPANDER OBJECT) ((TAG-AND-STREAM CONS) |&BODY| (BODY CONS)))", ((cpp_function_code)(&with_xml_tag_expander)), NULL);
    define_function_object("TERMINATE-OUTPUT-STREAM?", "(DEFUN (TERMINATE-OUTPUT-STREAM? BOOLEAN) ((SELF OUTPUT-STREAM)))", ((cpp_function_code)(&terminate_output_streamP)), NULL);
    define_function_object("TERMINATE-INPUT-STREAM?", "(DEFUN (TERMINATE-INPUT-STREAM? BOOLEAN) ((SELF INPUT-STREAM)))", ((cpp_function_code)(&terminate_input_streamP)), NULL);
    define_function_object("INITIALIZE-FILE-OUTPUT-STREAM", "(DEFUN INITIALIZE-FILE-OUTPUT-STREAM ((SELF FILE-OUTPUT-STREAM)))", ((cpp_function_code)(&initialize_file_output_stream)), NULL);
    define_function_object("TERMINATE-FILE-OUTPUT-STREAM?", "(DEFUN (TERMINATE-FILE-OUTPUT-STREAM? BOOLEAN) ((SELF FILE-OUTPUT-STREAM)))", ((cpp_function_code)(&terminate_file_output_streamP)), NULL);
    define_function_object("INITIALIZE-FILE-INPUT-STREAM", "(DEFUN INITIALIZE-FILE-INPUT-STREAM ((SELF FILE-INPUT-STREAM)))", ((cpp_function_code)(&initialize_file_input_stream)), NULL);
    define_function_object("TERMINATE-FILE-INPUT-STREAM?", "(DEFUN (TERMINATE-FILE-INPUT-STREAM? BOOLEAN) ((SELF FILE-INPUT-STREAM)))", ((cpp_function_code)(&terminate_file_input_streamP)), NULL);
    define_function_object("INITIALIZE-STRING-OUTPUT-STREAM", "(DEFUN INITIALIZE-STRING-OUTPUT-STREAM ((SELF STRING-OUTPUT-STREAM)))", ((cpp_function_code)(&initialize_string_output_stream)), NULL);
    define_method_object("(DEFMETHOD (THE-STRING-READER STRING) ((SELF STRING-OUTPUT-STREAM)))", ((cpp_method_code)(&Output_String_Stream::the_string_reader)), ((cpp_method_code)(NULL)));
    define_function_object("INITIALIZE-STRING-INPUT-STREAM", "(DEFUN INITIALIZE-STRING-INPUT-STREAM ((SELF STRING-INPUT-STREAM)))", ((cpp_function_code)(&initialize_string_input_stream)), NULL);
    define_function_object("CLOSE-STREAM", "(DEFUN CLOSE-STREAM ((SELF STREAM)) :DOCUMENTATION \"Close the stream `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&close_stream)), NULL);
    define_function_object("FLUSH-OUTPUT", "(DEFUN FLUSH-OUTPUT ((SELF OUTPUT-STREAM)) :DOCUMENTATION \"Flush all buffered output of `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&flush_output)), NULL);
    define_function_object("CLOSE-ALL-FILES", "(DEFUN CLOSE-ALL-FILES () :DOCUMENTATION \"Close all currently open file streams.  Use for emergencies or for cleanup.\" :PUBLIC? TRUE)", ((cpp_function_code)(&close_all_files)), NULL);
    define_function_object("OPEN-NETWORK-STREAM", "(DEFUN (OPEN-NETWORK-STREAM INPUT-STREAM OUTPUT-STREAM) ((HOST STRING) (PORT INTEGER)) :DOCUMENTATION \"Open a TCP/IP network stream to `host' at `port' and return the result\nas an input/output stream pair.\" :PUBLIC? TRUE)", ((cpp_function_code)(&open_network_stream)), NULL);
    define_function_object("HTML-CHARACTER-NEEDS-QUOTING?", "(DEFUN (HTML-CHARACTER-NEEDS-QUOTING? BOOLEAN) ((CH CHARACTER)))", ((cpp_function_code)(&html_character_needs_quotingP)), NULL);
    define_function_object("WRITE-HTML-CHARACTER-QUOTING-SPECIAL-CHARACTERS", "(DEFUN WRITE-HTML-CHARACTER-QUOTING-SPECIAL-CHARACTERS ((STREAM NATIVE-OUTPUT-STREAM) (CH CHARACTER)) :GLOBALLY-INLINE? TRUE (CASE CH (#\\> (PRINT-STREAM STREAM \"&gt;\")) (#\\< (PRINT-STREAM STREAM \"&lt;\")) (#\\& (PRINT-STREAM STREAM \"&amp;\")) (#\\\" (PRINT-STREAM STREAM \"&quot;\")) (OTHERWISE (PRINT-STREAM STREAM CH))) (RETURN))", ((cpp_function_code)(&write_html_character_quoting_special_characters)), NULL);
    define_function_object("WRITE-HTML-QUOTING-SPECIAL-CHARACTERS", "(DEFUN WRITE-HTML-QUOTING-SPECIAL-CHARACTERS ((STREAM NATIVE-OUTPUT-STREAM) (INPUT STRING)))", ((cpp_function_code)(&write_html_quoting_special_characters)), NULL);
    define_function_object("WRITE-HTML-ESCAPING-URL-SPECIAL-CHARACTERS", "(DEFUN WRITE-HTML-ESCAPING-URL-SPECIAL-CHARACTERS ((STREAM NATIVE-OUTPUT-STREAM) (INPUT STRING)))", ((cpp_function_code)(&write_html_escaping_url_special_characters)), NULL);
    define_function_object("UNESCAPE-HTML-STRING", "(DEFUN (UNESCAPE-HTML-STRING STRING) ((INPUT STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Replaces HTML escape sequences such as &amp; with their associated\ncharacters.\")", ((cpp_function_code)(&unescape_html_string)), NULL);
    define_function_object("TERMINATE-STREAM-ITERATOR?", "(DEFUN (TERMINATE-STREAM-ITERATOR? BOOLEAN) ((SELF STREAM-ITERATOR)))", ((cpp_function_code)(&terminate_stream_iteratorP)), NULL);
    define_function_object("S-EXPRESSIONS", "(DEFUN (S-EXPRESSIONS S-EXPRESSION-ITERATOR) ((STREAM INPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&s_expressions)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF S-EXPRESSION-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&S_Expression_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_function_object("LINES", "(DEFUN (LINES LINE-ITERATOR) ((STREAM INPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&lines)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF LINE-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Line_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_function_object("CHARACTERS", "(DEFUN (CHARACTERS CHARACTER-ITERATOR) ((STREAM INPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&characters)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF CHARACTER-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Character_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_function_object("STARTUP-STREAMS", "(DEFUN STARTUP-STREAMS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_streams)), NULL);
    { Method_Slot* function = lookup_function(SYM_STREAMS_STELLA_STARTUP_STREAMS);

      set_dynamic_slot_value(function->dynamic_slots, SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Streams"), NULL_STRING_WRAPPER);
    }
  }
}

void startup_streams() {
  if (current_startup_time_phaseP(0)) {
    { Output_Stream* self_001 = new_output_stream();

      self_001->native_stream = &cout;
      STANDARD_OUTPUT = self_001;
    }
    if (!(((boolean)(STANDARD_WARNING)))) {
      { Output_Stream* self_002 = new_output_stream();

        self_002->native_stream = &cerr;
        STANDARD_WARNING = self_002;
      }
    }
    { Output_Stream* self_003 = new_output_stream();

      self_003->native_stream = &cerr;
      STANDARD_ERROR = self_003;
    }
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_streams1();
      help_startup_streams2();
    }
    if (current_startup_time_phaseP(4)) {
      EOL = SYM_STREAMS_STELLA_EOL;
      { Input_Stream* self_000 = new_input_stream();

        self_000->native_stream = &cin;
        STANDARD_INPUT = self_000;
      }
      SINGLE_QUOTE_STRING = wrap_string("'");
      SINGLE_BQUOTE_STRING = wrap_string("`");
      oOPEN_FILE_STREAMSo = new_list();
      oHTML_ESCAPE_TABLEo = new_key_value_list();
    }
    if (current_startup_time_phaseP(5)) {
      { Class* renamed_Class = define_class_from_stringified_source("STREAM-ITERATOR", "(DEFCLASS STREAM-ITERATOR (ABSTRACT-ITERATOR) :PUBLIC? TRUE :ABSTRACT? TRUE :DOCUMENTATION \"Iterator that yields tokens from an input stream.\" :SLOTS ((THE-STREAM :TYPE INPUT-STREAM)) :TERMINATOR TERMINATE-STREAM-ITERATOR?)");

        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_stream_iterator_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("S-EXPRESSION-ITERATOR", "(DEFCLASS S-EXPRESSION-ITERATOR (STREAM-ITERATOR) :PUBLIC? TRUE :DOCUMENTATION \"Iterator that yields s-expressions from an input stream.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_s_expression_iterator));
      }
      { Class* renamed_Class = define_class_from_stringified_source("LINE-ITERATOR", "(DEFCLASS LINE-ITERATOR (STREAM-ITERATOR) :PUBLIC? TRUE :DOCUMENTATION \"Iterator that yields lines from an input stream.\" :PARAMETERS ((ANY-VALUE :TYPE STRING)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_line_iterator));
      }
      { Class* renamed_Class = define_class_from_stringified_source("CHARACTER-ITERATOR", "(DEFCLASS CHARACTER-ITERATOR (STREAM-ITERATOR) :PUBLIC? TRUE :DOCUMENTATION \"Iterator that yields characters from an input stream.\" :PARAMETERS ((ANY-VALUE :TYPE CHARACTER)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_character_iterator));
      }
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_streams3();
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL EOL SYMBOL (QUOTE EOL) :DOCUMENTATION \"Generates a newline character when passed to a stream.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL EOL-STRING STRING \"\n\" :DOCUMENTATION \"A string constant containing the character sequence\nnecessary to generate a newline.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL STANDARD-INPUT INPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard input stream for the host language.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL STANDARD-OUTPUT OUTPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard output stream for the host language.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL STANDARD-WARNING OUTPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard warning stream for the host language.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL STANDARD-ERROR OUTPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard error stream for the host language.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL SINGLE-QUOTE-STRING STRING-WRAPPER (WRAP-LITERAL \"'\") :DOCUMENTATION \"Holds a string containing the single quote character.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL SINGLE-BQUOTE-STRING STRING-WRAPPER (WRAP-LITERAL \"`\") :DOCUMENTATION \"Holds a string containing the single backquote character.\")");
      store_macro(SYM_STREAMS_STELLA_PRINT, SYM_STREAMS_STELLA_PRINT_EXPANDER, ((cpp_function_code)(&print_expander)));
      store_macro(SYM_STREAMS_STELLA_INFORM, SYM_STREAMS_STELLA_INFORM_EXPANDER, ((cpp_function_code)(&inform_expander)));
      store_macro(SYM_STREAMS_STELLA_WARN, SYM_STREAMS_STELLA_WARN_EXPANDER, ((cpp_function_code)(&warn_expander)));
      store_macro(SYM_STREAMS_STELLA_SIGNAL, SYM_STREAMS_STELLA_SIGNAL_EXPANDER, ((cpp_function_code)(&signal_expander)));
      store_macro(SYM_STREAMS_STELLA_ERROR, SYM_STREAMS_STELLA_ERROR_EXPANDER, ((cpp_function_code)(&error_expander)));
      store_macro(SYM_STREAMS_STELLA_CONTINUABLE_ERROR, SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER, ((cpp_function_code)(&continuable_error_expander)));
      store_macro(SYM_STREAMS_STELLA_SIGNAL_READ_ERROR, SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER, ((cpp_function_code)(&signal_read_error_expander)));
      store_macro(SYM_STREAMS_STELLA_WALK_ERROR, SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER, ((cpp_function_code)(&walk_error_expander)));
      store_macro(SYM_STREAMS_STELLA_WALK_INFORM, SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER, ((cpp_function_code)(&walk_inform_expander)));
      store_macro(SYM_STREAMS_STELLA_WALK_WARN, SYM_STREAMS_STELLA_WALK_WARN_EXPANDER, ((cpp_function_code)(&walk_warn_expander)));
      store_macro(SYM_STREAMS_STELLA_SAFETY, SYM_STREAMS_STELLA_SAFETY_EXPANDER, ((cpp_function_code)(&safety_expander)));
      store_macro(SYM_STREAMS_STELLA_EVALUATION_ERROR, SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER, ((cpp_function_code)(&evaluation_error_expander)));
      store_macro(SYM_STREAMS_STELLA_MESSAGE_TO_STRING, SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER, ((cpp_function_code)(&message_to_string_expander)));
      store_macro(SYM_STREAMS_STELLA_WITH_HTML_TAG, SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER, ((cpp_function_code)(&with_html_tag_expander)));
      store_macro(SYM_STREAMS_STELLA_WITH_XML_TAG, SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER, ((cpp_function_code)(&with_xml_tag_expander)));
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *OPEN-FILE-STREAMS* (LIST OF STREAM) (NEW (LIST OF STREAM)) :DOCUMENTATION \"List of file streams that are currently open.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *HTML-QUOTED-CHARACTERS* STRING \"><&\\\"\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *HTML-ESCAPE-TABLE* KEY-VALUE-LIST (NEW KEY-VALUE-LIST))");
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("lt"), wrap_character('<'));
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("gt"), wrap_character('>'));
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("amp"), wrap_character('&'));
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("quot"), wrap_character('"'));
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("apos"), wrap_character('\''));
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("nbsp"), wrap_character(' '));
      oHTML_ESCAPE_TABLEo->insert_at(wrap_string("NBSP"), wrap_character(' '));
    }
  }
}

Symbol* SYM_STREAMS_STELLA_EOL = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT_STREAM = NULL;

Symbol* SYM_STREAMS_STELLA_STANDARD_OUTPUT = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_INFORM = NULL;

Symbol* SYM_STREAMS_STELLA_INFORM_EXPANDER = NULL;

Keyword* KWD_STREAMS_COMMON_LISP = NULL;

Keyword* KWD_STREAMS_WARN = NULL;

Symbol* SYM_STREAMS_STELLA_STANDARD_WARNING = NULL;

Symbol* SYM_STREAMS_STELLA_WARN = NULL;

Symbol* SYM_STREAMS_STELLA_WARN_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_NULL = NULL;

Symbol* SYM_STREAMS_STELLA_STELLA_EXCEPTION = NULL;

Surrogate* SGT_STREAMS_STELLA_STRING_WRAPPER = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_EXCEPTION = NULL;

Symbol* SYM_STREAMS_STELLA_NEW = NULL;

Keyword* KWD_STREAMS_MESSAGE = NULL;

Symbol* SYM_STREAMS_STELLA_LET = NULL;

Symbol* SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM = NULL;

Symbol* SYM_STREAMS_STELLA_THE_STRING = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_ERROR_EXPANDER = NULL;

Keyword* KWD_STREAMS_JAVA = NULL;

Symbol* SYM_STREAMS_STELLA_ERROR_MESSAGE_ = NULL;

Symbol* SYM_STREAMS_STELLA_VERBATIM = NULL;

Keyword* KWD_STREAMS_CONTINUABLE_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_CONTINUABLE_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_READ_EXCEPTION = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_READ_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE = NULL;

Symbol* SYM_STREAMS_STELLA_SPECIAL = NULL;

Symbol* SYM_STREAMS_STELLA_oPRINTREADABLYpo = NULL;

Symbol* SYM_STREAMS_STELLA_TRUE = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_UNLESS = NULL;

Symbol* SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT = NULL;

Symbol* SYM_STREAMS_STELLA_STANDARD_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_INFORM = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_WARN = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_WARN_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_WHEN = NULL;

Symbol* SYM_STREAMS_STELLA_NOT = NULL;

Keyword* KWD_STREAMS_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_SAFETY = NULL;

Symbol* SYM_STREAMS_STELLA_SAFETY_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_oEVALUATIONTREEo = NULL;

Symbol* SYM_STREAMS_STELLA_DEFINEDp = NULL;

Symbol* SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo = NULL;

Symbol* SYM_STREAMS_STELLA_EVALUATION_EXCEPTION = NULL;

Symbol* SYM_STREAMS_STELLA_EVALUATION_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER = NULL;

Surrogate* SGT_STREAMS_STELLA_LITERAL_WRAPPER = NULL;

Symbol* SYM_STREAMS_STELLA_STRINGIFY = NULL;

Symbol* SYM_STREAMS_STELLA_VRLET = NULL;

Symbol* SYM_STREAMS_STELLA_STRING = NULL;

Symbol* SYM_STREAMS_STELLA_SETQ = NULL;

Symbol* SYM_STREAMS_STELLA_CONCATENATE = NULL;

Symbol* SYM_STREAMS_STELLA_MESSAGE_TO_STRING = NULL;

Symbol* SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER = NULL;

Surrogate* SGT_STREAMS_STELLA_OBJECT = NULL;

Symbol* SYM_STREAMS_STELLA_PROGN = NULL;

Symbol* SYM_STREAMS_STELLA_SUBSEQUENCE = NULL;

Symbol* SYM_STREAMS_STELLA_POSITION = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_HTML_TAG = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_XML_TAG = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER = NULL;

Keyword* KWD_STREAMS_CLOSED = NULL;

Keyword* KWD_STREAMS_OPEN = NULL;

Surrogate* SGT_STREAMS_STELLA_FILE_INPUT_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_INPUT_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_OUTPUT_STREAM = NULL;

Symbol* SYM_STREAMS_STELLA_THE_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR = NULL;

Surrogate* SGT_STREAMS_STELLA_LINE_ITERATOR = NULL;

Surrogate* SGT_STREAMS_STELLA_CHARACTER_ITERATOR = NULL;

Symbol* SYM_STREAMS_STELLA_STARTUP_STREAMS = NULL;

Symbol* SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
